'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.NOT_ITERATOR_ERROR = undefined;
exports.default = proc;

var _utils = require('./utils');

var _io = require('./io');

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var NOT_ITERATOR_ERROR = exports.NOT_ITERATOR_ERROR = "proc argument must be an iterator";

function proc(iterator) {
  var subscribe = arguments.length <= 1 || arguments[1] === undefined ? function () {
    return function () {};
  } : arguments[1];
  var dispatch = arguments.length <= 2 || arguments[2] === undefined ? function () {} : arguments[2];

  if (!_utils.is.iterator(iterator)) throw new Error(NOT_ITERATOR_ERROR);

  var deferredInput = undefined,
      deferredEnd = undefined;
  var canThrow = _utils.is.throw(iterator);

  var endP = new Promise(function (resolve, reject) {
    return deferredEnd = { resolve: resolve, reject: reject };
  });

  var unsubscribe = subscribe(function (input) {
    if (deferredInput && deferredInput.match(input)) deferredInput.resolve(input);
  });

  next();
  return endP;

  function next(arg, isError) {
    deferredInput = null;
    try {
      if (isError && !canThrow) throw arg;
      var result = isError ? iterator.throw(arg) : iterator.next(arg);

      if (!result.done) runEffect(result.value).then(next, function (err) {
        return next(err, true);
      });else {
        unsubscribe();
        deferredEnd.resolve(result.value);
      }
    } catch (err) {
      unsubscribe();
      deferredEnd.reject(err);
    }
  }

  function runEffect(effect) {
    var _data;

    var data = undefined;
    return _utils.is.promise(effect) ? effect : _utils.is.array(effect) ? Promise.all(effect.map(runEffect)) : _utils.is.iterator(effect) ? proc(effect, subscribe, dispatch) : (data = _io.as.take(effect)) ? runTakeEffect(data) : (data = _io.as.put(effect)) ? Promise.resolve(dispatch(data)) : (data = _io.as.race(effect)) ? runRaceEffect(data) : (data = _io.as.call(effect)) ? (_data = data).fn.apply(_data, _toConsumableArray(data.args)) : (data = _io.as.cps(effect)) ? runCPSEffect(data) : /* resolve anything else  */Promise.resolve(effect);
  }

  function runTakeEffect(pattern) {
    return new Promise(function (resolve) {
      deferredInput = { resolve: resolve, match: (0, _io.matcher)(pattern), pattern: pattern };
    });
  }

  function runCPSEffect(cps) {
    return new Promise(function (resolve, reject) {
      cps.fn.apply(cps, [].concat(_toConsumableArray(cps.args), [function (err, res) {
        return _utils.is.undef(err) ? resolve(res) : reject(err);
      }]));
    });
  }

  function runRaceEffect(effects) {
    return Promise.race(Object.keys(effects).map(function (key) {
      return runEffect(effects[key]).then(function (result) {
        return _defineProperty({}, key, result);
      }, function (error) {
        return Promise.reject(_defineProperty({}, key, error));
      });
    }));
  }
}